@@ -1,9 +1,8 @@
 use crate::{
     config::{Config as GeneralConfig, DelaySchedule},
-    nodes::{NodeCount, NodeIndex},
     runway::NotificationOut,
     units::{PreUnit, Unit},
-    Hasher, Receiver, Round, Sender,
+    Hasher, NodeCount, NodeIndex, Receiver, Round, Sender,
 };
 use futures::{channel::oneshot, FutureExt, StreamExt};
 use futures_timer::Delay;
@@ -12,7 +11,7 @@ use std::time::Duration;
 
 mod creator;
 
-use creator::Creator;
+pub use creator::Creator;
 
 /// The configuration needed for the process creating new units.
 pub struct Config {
