@@ -1,55 +1,12 @@
 use crate::{
-    alerts::AlertMessage,
-    member::UnitMessage,
-    nodes::NodeIndex,
-    signed::{PartialMultisignature, Signature},
-    Data, Hasher, Receiver, Sender,
+    alerts::AlertMessage, member::UnitMessage, Data, Hasher, Network, PartialMultisignature,
+    Receiver, Recipient, Sender, Signature,
 };
 use codec::{Decode, Encode};
 use futures::{channel::oneshot, FutureExt, StreamExt};
 use log::{error, info, warn};
 use std::fmt::Debug;
 
-/// A recipient of a message, either a specific node or everyone.
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub enum Recipient {
-    Everyone,
-    Node(NodeIndex),
-}
-
-/// Network represents an interface for sending and receiving NetworkData.
-///
-/// Note on Rate Control: it is assumed that Network implements a rate control mechanism guaranteeing
-/// that no node is allowed to spam messages without limits. We do not specify details yet, but in
-/// future releases we plan to publish recommended upper bounds for the amounts of bandwidth and
-/// number of messages allowed per node per a unit of time. These bounds must be carefully crafted
-/// based upon the number of nodes N and the configured delays between subsequent Dag rounds, so
-/// that at the same time spammers are cut off but honest nodes are able function correctly within
-/// these bounds.
-///
-/// Note on Network Reliability: it is not assumed that each message that AlephBFT orders to send
-/// reaches its intended recipient, there are some built-in reliability mechanisms within AlephBFT
-/// that will automatically detect certain failures and resend messages as needed. Clearly, the less
-/// reliable the network is, the worse the performarmence of AlephBFT will be (generally slower to
-/// produce output). Also, not surprisingly if the percentage of dropped messages is too high
-/// AlephBFT might stop making progress, but from what we observe in tests, this happens only when
-/// the reliability is extremely bad, i.e., drops below 50% (which means there is some significant
-/// issue with the network).
-///
-/// We refer to the documentation https://cardinal-cryptography.github.io/AlephBFT/aleph_bft_api.html
-/// Section 3.1.2 for a discussion of the required guarantees of this trait's implementation.
-#[async_trait::async_trait]
-pub trait Network<H: Hasher, D: Data, S: Signature, MS: PartialMultisignature>: Send {
-    /// Send a message to a single node or everyone, depending on the value of the recipient
-    /// argument.
-    ///
-    /// Note on the implementation: this function should be implemented in a non-blocking manner.
-    /// Otherwise, the performance might be affected negatively or the execution may end up in a deadlock.
-    fn send(&self, data: NetworkData<H, D, S, MS>, recipient: Recipient);
-    /// Receive a message from the network.
-    async fn next_event(&mut self) -> Option<NetworkData<H, D, S, MS>>;
-}
-
 #[derive(Encode, Decode, Clone, Debug)]
 pub(crate) enum NetworkDataInner<H: Hasher, D: Data, S: Signature, MS: PartialMultisignature> {
     Units(UnitMessage<H, D, S>),
@@ -114,7 +71,7 @@ struct NetworkHub<
     D: Data,
     S: Signature,
     MS: PartialMultisignature,
-    N: Network<H, D, S, MS>,
+    N: Network<NetworkData<H, D, S, MS>>,
 > {
     network: N,
     units_to_send: Receiver<(UnitMessage<H, D, S>, Recipient)>,
@@ -123,8 +80,13 @@ struct NetworkHub<
     alerts_received: Sender<AlertMessage<H, D, S, MS>>,
 }
 
-impl<H: Hasher, D: Data, S: Signature, MS: PartialMultisignature, N: Network<H, D, S, MS>>
-    NetworkHub<H, D, S, MS, N>
+impl<
+        H: Hasher,
+        D: Data,
+        S: Signature,
+        MS: PartialMultisignature,
+        N: Network<NetworkData<H, D, S, MS>>,
+    > NetworkHub<H, D, S, MS, N>
 {
     fn new(
         network: N,
@@ -201,7 +163,7 @@ pub(crate) async fn run<
     D: Data,
     S: Signature,
     MS: PartialMultisignature,
-    N: Network<H, D, S, MS>,
+    N: Network<NetworkData<H, D, S, MS>>,
 >(
     network: N,
     units_to_send: Receiver<(UnitMessage<H, D, S>, Recipient)>,
@@ -223,39 +185,49 @@ pub(crate) async fn run<
 
 #[cfg(test)]
 mod tests {
-    use super::*;
     use crate::{
-        nodes::NodeSubset,
-        testing::mock::{self, Data, Hasher64, PartialMultisignature, Signature},
+        alerts::AlertMessage,
+        member::UnitMessage,
+        network::NetworkDataInner::{Alert, Units},
         units::{ControlHash, FullUnit, PreUnit, UncheckedSignedUnit, UnitCoord},
-        Round, UncheckedSigned,
+        Hasher, NodeIndex, NodeSubset, Round, Signed,
     };
+    use aleph_bft_mock::{Data, Hasher64, Keychain, PartialMultisignature, Signature};
+    use codec::{Decode, Encode};
 
-    fn test_unchecked_unit(
+    async fn test_unchecked_unit(
         creator: NodeIndex,
         round: Round,
-        variant: u32,
+        data: Data,
     ) -> UncheckedSignedUnit<Hasher64, Data, Signature> {
         let control_hash = ControlHash {
             parents_mask: NodeSubset::with_size(7.into()),
             combined_hash: 0.using_encoded(Hasher64::hash),
         };
         let pu = PreUnit::new(creator, round, control_hash);
-        let data = Data::new(UnitCoord::new(7, 13.into()), variant);
-        UncheckedSigned::new(FullUnit::new(pu, data, 0), Signature {})
+        let signable = FullUnit::new(pu, data, 0);
+        Signed::sign(signable, &Keychain::new(0.into(), creator))
+            .await
+            .into_unchecked()
     }
 
-    #[test]
-    fn decoding_network_data_units_new_unit() {
-        use NetworkDataInner::Units;
+    type TestNetworkData = super::NetworkData<Hasher64, Data, Signature, PartialMultisignature>;
+    impl TestNetworkData {
+        fn new(
+            inner: super::NetworkDataInner<Hasher64, Data, Signature, PartialMultisignature>,
+        ) -> Self {
+            super::NetworkData::<Hasher64, Data, Signature, PartialMultisignature>(inner)
+        }
+    }
+
+    #[tokio::test]
+    async fn decoding_network_data_units_new_unit() {
         use UnitMessage::NewUnit;
 
-        let uu = test_unchecked_unit(5.into(), 43, 1729);
-        let included_data = vec![uu.as_signable().data().clone()];
-        let nd = NetworkData::<Hasher64, Data, Signature, PartialMultisignature>(Units(NewUnit(
-            uu.clone(),
-        )));
-        let decoded = mock::NetworkData::decode(&mut &nd.encode()[..]);
+        let uu = test_unchecked_unit(5.into(), 43, 1729).await;
+        let included_data = vec![*uu.as_signable().data()];
+        let nd = TestNetworkData::new(Units(NewUnit(uu.clone())));
+        let decoded = TestNetworkData::decode(&mut &nd.encode()[..]);
         assert!(decoded.is_ok(), "Bug in encode/decode for NewUnit");
         let decoded = decoded.unwrap();
         assert_eq!(
@@ -276,15 +248,12 @@ mod tests {
 
     #[test]
     fn decoding_network_data_units_request_coord() {
-        use NetworkDataInner::Units;
         use UnitMessage::RequestCoord;
 
         let ni = 7.into();
         let uc = UnitCoord::new(3, 13.into());
-        let nd = NetworkData::<Hasher64, Data, Signature, PartialMultisignature>(Units(
-            RequestCoord(ni, uc),
-        ));
-        let decoded = mock::NetworkData::decode(&mut &nd.encode()[..]);
+        let nd = TestNetworkData::new(Units(RequestCoord(ni, uc)));
+        let decoded = TestNetworkData::decode(&mut &nd.encode()[..]);
         assert!(decoded.is_ok(), "Bug in encode/decode for RequestCoord");
         let decoded = decoded.unwrap();
         assert!(
@@ -298,17 +267,14 @@ mod tests {
         }
     }
 
-    #[test]
-    fn decoding_network_data_units_response_coord() {
-        use NetworkDataInner::Units;
+    #[tokio::test]
+    async fn decoding_network_data_units_response_coord() {
         use UnitMessage::ResponseCoord;
 
-        let uu = test_unchecked_unit(5.into(), 43, 1729);
-        let included_data = vec![uu.as_signable().data().clone()];
-        let nd = NetworkData::<Hasher64, Data, Signature, PartialMultisignature>(Units(
-            ResponseCoord(uu.clone()),
-        ));
-        let decoded = mock::NetworkData::decode(&mut &nd.encode()[..]);
+        let uu = test_unchecked_unit(5.into(), 43, 1729).await;
+        let included_data = vec![*uu.as_signable().data()];
+        let nd = TestNetworkData::new(Units(ResponseCoord(uu.clone())));
+        let decoded = TestNetworkData::decode(&mut &nd.encode()[..]);
         assert!(decoded.is_ok(), "Bug in encode/decode for ResponseCoord");
         let decoded = decoded.unwrap();
         assert_eq!(
@@ -329,15 +295,12 @@ mod tests {
 
     #[test]
     fn decoding_network_data_units_request_parents() {
-        use NetworkDataInner::Units;
         use UnitMessage::RequestParents;
 
         let ni = 7.into();
         let h = 43.using_encoded(Hasher64::hash);
-        let nd = NetworkData::<Hasher64, Data, Signature, PartialMultisignature>(Units(
-            RequestParents(ni, h),
-        ));
-        let decoded = mock::NetworkData::decode(&mut &nd.encode()[..]);
+        let nd = TestNetworkData::new(Units(RequestParents(ni, h)));
+        let decoded = TestNetworkData::decode(&mut &nd.encode()[..]);
         assert!(decoded.is_ok(), "Bug in encode/decode for RequestParents");
         let decoded = decoded.unwrap();
         assert!(
@@ -351,26 +314,23 @@ mod tests {
         }
     }
 
-    #[test]
-    fn decoding_network_data_units_response_parents() {
-        use NetworkDataInner::Units;
+    #[tokio::test]
+    async fn decoding_network_data_units_response_parents() {
         use UnitMessage::ResponseParents;
 
         let h = 43.using_encoded(Hasher64::hash);
-        let p1 = test_unchecked_unit(5.into(), 43, 1729);
-        let p2 = test_unchecked_unit(13.into(), 43, 1729);
-        let p3 = test_unchecked_unit(17.into(), 43, 1729);
+        let p1 = test_unchecked_unit(5.into(), 43, 1729).await;
+        let p2 = test_unchecked_unit(13.into(), 43, 1729).await;
+        let p3 = test_unchecked_unit(17.into(), 43, 1729).await;
         let included_data = vec![
-            p1.as_signable().data().clone(),
-            p2.as_signable().data().clone(),
-            p3.as_signable().data().clone(),
+            *p1.as_signable().data(),
+            *p2.as_signable().data(),
+            *p3.as_signable().data(),
         ];
         let parents = vec![p1, p2, p3];
 
-        let nd = NetworkData::<Hasher64, Data, Signature, PartialMultisignature>(Units(
-            ResponseParents(h, parents.clone()),
-        ));
-        let decoded = mock::NetworkData::decode(&mut &nd.encode()[..]);
+        let nd = TestNetworkData::new(Units(ResponseParents(h, parents.clone())));
+        let decoded = TestNetworkData::decode(&mut &nd.encode()[..]);
         assert!(decoded.is_ok(), "Bug in encode/decode for ResponseParents");
         let decoded = decoded.unwrap();
         assert_eq!(
@@ -397,26 +357,25 @@ mod tests {
         }
     }
 
-    #[test]
-    fn decoding_network_data_alert_fork_alert() {
+    #[tokio::test]
+    async fn decoding_network_data_alert_fork_alert() {
         use AlertMessage::ForkAlert;
-        use NetworkDataInner::Alert;
 
         let forker = 9.into();
-        let f1 = test_unchecked_unit(forker, 10, 0);
-        let f2 = test_unchecked_unit(forker, 10, 1);
-        let lu1 = test_unchecked_unit(forker, 11, 0);
-        let lu2 = test_unchecked_unit(forker, 12, 0);
-        let included_data = vec![
-            lu1.as_signable().data().clone(),
-            lu2.as_signable().data().clone(),
-        ];
-        let alert = crate::alerts::Alert::new(7.into(), (f1, f2), vec![lu1, lu2]);
+        let f1 = test_unchecked_unit(forker, 10, 0).await;
+        let f2 = test_unchecked_unit(forker, 10, 1).await;
+        let lu1 = test_unchecked_unit(forker, 11, 0).await;
+        let lu2 = test_unchecked_unit(forker, 12, 0).await;
+        let included_data = vec![*lu1.as_signable().data(), *lu2.as_signable().data()];
+        let sender: NodeIndex = 7.into();
+        let alert = crate::alerts::Alert::new(sender, (f1, f2), vec![lu1, lu2]);
 
-        let nd = NetworkData::<Hasher64, Data, Signature, PartialMultisignature>(Alert(ForkAlert(
-            UncheckedSigned::new(alert.clone(), Signature {}),
+        let nd = TestNetworkData::new(Alert(ForkAlert(
+            Signed::sign(alert.clone(), &Keychain::new(0.into(), sender))
+                .await
+                .into_unchecked(),
         )));
-        let decoded = mock::NetworkData::decode(&mut &nd.encode()[..]);
+        let decoded = TestNetworkData::decode(&mut &nd.encode()[..]);
         assert!(decoded.is_ok(), "Bug in encode/decode for ForkAlert");
         let decoded = decoded.unwrap();
         assert_eq!(
