@@ -1,9 +1,11 @@
-use futures::StreamExt;
-
 use crate::{
-    testing::mock::{configure_network, init_log, spawn_honest_member, Spawner},
+    testing::{init_log, spawn_honest_member},
     NodeCount, SpawnHandle,
 };
+use aleph_bft_mock::{Router, Spawner};
+use futures::StreamExt;
+use parking_lot::Mutex;
+use std::sync::Arc;
 
 async fn honest_members_agree_on_batches(
     n_members: NodeCount,
@@ -16,14 +18,19 @@ async fn honest_members_agree_on_batches(
     let mut exits = Vec::new();
     let mut handles = Vec::new();
     let mut batch_rxs = Vec::new();
-    let (net_hub, networks) = configure_network(n_members, network_reliability);
+    let (net_hub, networks) = Router::new(n_members, network_reliability);
     spawner.spawn("network-hub", net_hub);
 
     for network in networks {
         let ix = network.index();
         if n_alive.into_range().contains(&ix) {
-            let (batch_rx, exit_tx, handle) =
-                spawn_honest_member(spawner.clone(), ix, n_members, network);
+            let (batch_rx, exit_tx, handle) = spawn_honest_member(
+                spawner.clone(),
+                ix,
+                n_members,
+                Arc::new(Mutex::new(vec![])),
+                network,
+            );
             batch_rxs.push(batch_rx);
             exits.push(exit_tx);
             handles.push(handle);
